use crate::{
    types,
    util::{junk, nfl, request},
};

use serenity::builder::{CreateCommand, CreateCommandOption};
use serenity::model::application::{CommandOptionType, ResolvedOption, ResolvedValue};

pub async fn run(options: &[ResolvedOption<'_>]) -> String {
    if let Some(ResolvedOption {
        value: ResolvedValue::String(team),
        ..
    }) = options.first()
    {
        let teams = nfl::nfl_teams();
        let team_id = match junk::find_team_key(&teams, team.to_lowercase().as_str()) {
            Some(t) => t,
            None => return junk::get_random_insult(),
        };
        let req = request::Request::new();
        let tmp_url = format!("{:}/nfl/teams/{:}", req.base_url, team_id);
        let tmp_result = req
            .request_url::<types::team::TeamResponse>(&tmp_url)
            .await
            .unwrap();
        let event_id = tmp_result.team.next_event[0].id.clone();
        let url = format!("{:}/nfl/scoreboard/{:}", req.base_url, event_id);
        let result = req
            .request_url::<types::event::EventResponse>(&url)
            .await
            .unwrap();
        let game = result.competitions[0].clone();

        let event_title = format!("{:} ({:})", result.name, result.short_name); // needed
        let event_date = format!(
            "<t:{:}:R>",
            junk::flexible_to_epoch_seconds(result.date.as_str()).unwrap()
        ); // needed
        let clock = result.status.display_clock.clone();
        let quarter = result.status.period.to_string().clone();
        let venue = game.venue.full_name.clone();
        let team_home = game
            .competitors
            .iter()
            .find(|t| t.home_away == "home")
            .unwrap();
        let team_away = game
            .competitors
            .iter()
            .find(|t| t.home_away == "away")
            .unwrap();
        let home_team = team_home.team.display_name.clone();
        let away_team = team_away.team.display_name.clone();
        let home_team_score = team_home.score.clone();
        let away_team_score = team_away.score.clone();

        let discord_response = GameDiscordResponse {
            event_title,
            home_team,
            home_team_score,
            away_team,
            away_team_score,
            venue,
            event_date,
            clock,
            quarter,
        };
        discord_response.format()
    } else {
        junk::get_random_insult()
    }
}

pub fn register() -> CreateCommand {
    CreateCommand::new("game")
        .description("Get the next game's information by specifying a team name")
        .add_option(
            CreateCommandOption::new(
                CommandOptionType::String,
                "team",
                "The team to query to get team's game info",
            )
            .required(true),
        )
}

pub struct GameDiscordResponse {
    pub event_title: String,
    pub home_team: String,
    pub home_team_score: String,
    pub away_team: String,
    pub away_team_score: String,
    pub venue: String,
    pub event_date: String,
    pub clock: String,
    pub quarter: String,
}

impl GameDiscordResponse {
    pub fn format(&self) -> String {
        format!(
            "\n## {:}\nGame Date: {:}\nHome Team: {:}\nHome Team Score: {:}\nAway Team: {:}\nAway Team Score: {:}\nVenue: {:}\nClock: {:}\nQuarter: {:}",
            self.event_title, self.event_date, self.home_team, self.home_team_score, self.away_team, self.away_team_score, self.venue, self.clock, self.quarter
        )
    }
}
